/*************** schema format from the backend *************** */

/** Schema type, consist of nodes and edges */
export type SchemaFromBackend = {
  edges: SchemaEdge[];
  nodes: SchemaNode[];
};

export type SchemaStatsFromBackend = {
  edgeStats: {
    type: string;
    count: number;
  }[];
  nodeStats: {
    key: string;
    labels: string[];
    count: number;
  }[];
};

export type SchemaAttributeTypes =
  | 'string'
  | 'float'
  | 'int'
  | 'bool'
  | 'boolean'
  | 'date'
  | 'time'
  | 'datetime'
  | 'duration'
  | 'number'
  | 'location';

export type DimensionType = 'categorical' | 'numerical' | 'temporal' | 'spatial';

/** Attribute type, consist of a name */
export type SchemaAttribute = {
  name: string;
  type: SchemaAttributeTypes;
  dimension?: DimensionType;
  uniqueCount?: number;
  categoryCounts?: Record<string, number>;
  minValue?: number | string;
  maxValue?: number | string;
  avgValue?: number;
  stDevValue?: number;
  firstDate?: string;
  lastDate?: string;
  [id: string]: any;
};

/** Node type, consist of a name and a list of attributes */
export type SchemaNode = {
  name: string;
  attributes: SchemaAttribute[];
  type?: string;
};

export type GraphAttributeStats = {
  nodeStats: Array<NodeOrEdgeStats>;
  edgeStats: Array<NodeOrEdgeStats>;
};

export type NodeOrEdgeStats = {
  key: string;
  type?: string;
  count: number;
  attributeStats: Array<AttributeStats>;
};

export type AttributeStats =
  | BooleanAttributeStats
  | StringAttributeStats
  | NumberAttributeStats
  | DateTimeAttributeStats
  | LocationAttributeStats;

export type CommonAttributeStats = {
  name: string;
  type: string;
};

export type BooleanAttributeStats = CommonAttributeStats & {
  type: 'boolean';
  uniqueCount: number;
  categoryCounts?: Record<string, number>;
};

export type StringAttributeStats = CommonAttributeStats & {
  type: 'string';
  uniqueCount: number;
  categoryCounts?: Record<string, number>;
};

export type NumberAttributeStats = CommonAttributeStats & {
  type: 'number';
  minValue: number;
  maxValue: number;
  avgValue: number;
  stDevValue: number;
};

export type DateTimeAttributeStats = CommonAttributeStats & {
  type: 'datetime';
  firstDate: string;
  lastDate: string;
};

export type LocationAttributeStats = CommonAttributeStats & {
  type: 'location';
  minValue: string;
  maxValue: string;
};

/** Edge type, consist of a name, start point, end point and a list of attributes */
export type SchemaEdge = {
  name: string;
  to: string;
  from: string;
  collection: string;
  attributes: SchemaAttribute[];
  label: string;
  type?: string;
};

export type GraphAttributeDimensions = {
  nodes: { [id: string]: { [id: string]: string } };
  edges: { [id: string]: { [id: string]: string } };
};
